import praw
import re
import requests
from datetime import datetime
from pprint import pprint
import myredditdl.utils as utils
from myredditdl.console_args import get_console_args as args


class Item:
    ''' This class will house the current Reddit item'''
    def __init__(self, item):
        self.log = utils.setup_logger(__name__, args()['debug'])
        self.__item = item
        self.__media_url = self._fetch_media_url()

    def __len__(self):
        return len(self.__media_url)

    def __getitem__(self):
        return self.__item

    def __repr__(self):
        return 'Item(id=%r, title=%r, link=%r, sub=%r author=%r)' % (self.get_id(),
                                                                     self.get_title(),
                                                                     self.get_reddit_link(),
                                                                     self.get_subreddit_prefixed(),
                                                                     self.get_author())

    def __str__(self):
        return self.fmt.format('-' * 50,
                               'Id', self.get_id(),
                               'Title', self.get_title(),
                               'Link', self.get_reddit_link(),
                               'Domain', self.get_domain(),
                               'Sub', self.get_subreddit_prefixed(),
                               'Author', self.get_author(),
                               'Video', self.is_video(),
                               '-' * 50)

    @property
    def __dict__(self):
        # call with vars(item) instead of pprint(vars(item))
        return pprint(vars(self.__item))

    @property
    def fmt(self):
        return ('{}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{:6} = {}\n'
                '{}\n')

    @property
    def __mapped_domains(self) -> dict:
        return {'v.redd.it': self.get_vreddit_url,
                'gfycat.com': self.get_gfycat_url,
                'redgifs.com': self.get_redgifs_url,
                'streamable.com': self.get_streamable_url,
                'imgur.com': self.get_imgur_url,
                'reddit.com': self.get_reddit_gallery_url
                }

    def get_item(self) -> 'RedditPostItem':
        return self.__item

    def get_title(self) -> str:
        ''' Reddit post title without unicode'''
        return str(self.__item.title).encode('ascii', 'ignore').decode()

    def get_author(self) -> str:
        ''' Reddit post author username'''
        return str(self.__item.author)

    def get_id(self) -> str:
        ''' Reddit post id number'''
        return str(self.__item.id)

    def get_domain(self) -> str:
        ''' Domain in which the media post is hosted'''
        return str(self.__item.domain)

    def get_url(self) -> str:
        ''' Url of the media on the domain is hosted.
            With images it almost always directs to a direct downloable image.
            But with videos, it directs to a non-downloadble source.

        '''
        return str(self.__item.url)

    def get_media_url(self) -> list:
        return self.__media_url

    def get_reddit_link(self) -> str:
        ''' Link of the Reddit post'''
        return 'https://reddit.com' + str(self.__item.permalink)

    def get_subreddit(self) -> str:
        ''' Subreddit name without the r/ prefix'''
        return str(self.__item.subreddit)

    def get_subreddit_prefixed(self) -> str:
        ''' Returns the item subreddit in the format r/Subreddit'''
        return str(self.__item.subreddit_name_prefixed)

    def get_upvotes_amount(self) -> str:
        return str(self.__item.ups)

    def is_nsfw(self) -> bool:
        return self.__item.over_18

    def is_valid_media_post(self) -> bool:
        if self.get_media_url():
            return True
        return False

    def is_video(self) -> bool:
        return self.__item.domain in utils.VIDEO_DOMAINS

    def is_comment(self) -> bool:
        return isinstance(self.__item, praw.models.reddit.comment.Comment)

    def get_creation_date(self) -> str:
        time_utc = self.__item.created_utc
        return str(datetime.fromtimestamp(time_utc).strftime('%m/%d/%Y'))

    def get_metadata(self) -> dict:
        return {'Author': self.get_author(),
                'Subreddit': self.get_subreddit_prefixed(),
                'Title': self.get_title(),
                'Link': self.get_reddit_link(),
                'Upvotes': self.get_upvotes_amount(),
                'NSFW': self.is_nsfw(),
                'Post creation date': self.get_creation_date()}

    def _fetch_media_url(self) -> list:
        ''' Returns a list of media url(s) for the given post item

            @params:
                domain: media domain of current post item (self.item)

                url:    link to the domain where the media is beign stored.
                        This link can't be used for download. Must extract
                        the media url for download.

            @return: list of url(s) containing the downloable media
        '''
        if self.__item.domain in self.__mapped_domains.keys():
            return self.__mapped_domains[self.__item.domain]()
        return [self.__item.url.replace('gifv', 'mp4').replace('gif', 'mp4')]

    def get_vreddit_url(self) -> list:
        ''' Extracts downloadable url for https://v.redd.it domain posts'''
        try:
            return [self.__item.media['reddit_video']['fallback_url']]

        except Exception:
            self.log.debug("vreddit_url: media['reddit_video'] exception")

        try:
            return [self.__item.crosspost_parent_list[0]
                    ['media']['reddit_video']['fallback_url']]

        except Exception:
            self.log.debug("vreddit_url: crosspost_parent_list exception")

        try:
            return [self.__item.media['oembed']
                    ['thumbnail_url'].replace('jpg', 'mp4')]

        except Exception:
            self.log.debug('v.redd.it media url not found. returning []')
            return []

    def get_gfycat_url(self) -> list:
        ''' Extracts downloadable url for https://gfycat.com domain posts'''
        try:
            return [self.__item.preview['reddit_video_preview']['fallback_url']]

        except Exception:
            self.log.debug(
                f'Gfycat_url: item preview exception: {self.__repr__()}')

        try:
            return [self.__item.media['oembed']
                    ['thumbnail_url'].replace('jpg', 'mp4')]

        except Exception:
            self.log.debug('Gfycat_url: media url not found. Returning []')
            return []

    def get_redgifs_url(self) -> list:
        ''' Extracts downloadable url for https://redgifs.com domain posts'''
        try:
            return [self.__item.preview['reddit_video_preview']['fallback_url']]

        except Exception:
            self.log.debug('Redgifs_url: item preview exception')

        try:
            return [self.__item.media['oembed']
                    ['thumbnail_url'].replace('jpg', 'mp4')]

        except Exception:
            self.log.debug(
                'Redgifs_url: item media oembed exception. Requesting data')

        # This is expensive. Only do it if completely neccesary
        # Extract the video link through html requests
        response = requests.get(self.__item.url).text
        urls = re.findall(r'https?://[^\s<>"]+|www\.[^\s<>"]+', str(response))
        return [url for url in urls if url.endswith('.mp4')][:1]

    def get_streamable_url(self) -> list:
        # TODO: debug this
        try:
            html = self.__item.media['oembed']['html']
            url = re.findall(r'https?://[^\s<>"]+|www\.[^\s<>"]+', str(html))
            return [url + '.mp4']
        except Exception:
            self.log.debug('streamable_url exception raised')
            return []

    def get_reddit_gallery_url(self) -> list:
        try:
            metadata = self.__item.media_metadata.values()
            return [i['s']['u'] for i in metadata if i['e'] == 'Image']
        except Exception:
            self.log.debug(
                'reddit_gallery_url: exception raised. post deleted')
            return []

    def get_imgur_url(self) -> list:
        try:
            return [self.__item.preview['images'][0]['source']['url']]
        except Exception:
            self.log.debug('imgur_url: item preview exception.')
        return [self.__item.url.replace('gifv', 'mp4').replace('gif', 'mp4')]



import pathlib
import os
import myredditdl.utils as utils
from myredditdl.config_handler import ConfigHandler


class Defaults:
    def __init__(self, debug=False) -> None:
        self.log = utils.setup_logger(__name__)
        self.config_handler = ConfigHandler()
        self.debug = debug

    @property
    def home_dir(self) -> str:
        return os.path.expanduser('~')

    @property
    def project_parent_dir(self) -> str:
        return str(pathlib.Path(__file__).parent.parent) + os.sep

    @property
    def src_dir(self) -> str:
        ''' Source files folder'''
        return str(pathlib.Path(__file__).parent) + os.sep

    @property
    def debug_media_dir(self) -> str:
        ''' Source files folder'''
        return str(pathlib.Path(__file__).parent) + \
            os.sep + 'debug_media' + os.sep

    @property
    def metadata_folder(self) -> str:
        return self.src_dir + 'metadata' + os.sep

    @property
    def debug_log_file(self) -> str:
        return self.src_dir + os.sep + 'debug.log'

    @property
    def debug_path(self) -> str:
        return str(self.project_parent_dir + 'debug_media' + os.sep)

    @property
    def metadata_suffix(self):
        return '_debug.json' if self.debug else '_metadata.json'

    @property
    def metadata_file(self) -> str:
        ''' Returns the full path of the metadata file'''
        username = self.config_handler.get_client_username()
        return self.metadata_folder + username + self.metadata_suffix

    @property
    def media_path(self):
        if self.debug:
            return self.debug_media_dir
        return self.config_handler.get_media_path()

    @property
    def current_prefix(self):
        return self.config_handler.get_prefix()



#!/usr/bin/env python3

"""
Author: Emanuel Ramirez Alsina (emanuel2718 @ Github)
Program: myredditdl
Description: Reddit upvoted & saved media downloader
"""
import sys
import myredditdl.console_args as console_args
from myredditdl.downloader import Downloader


def run():
    if len(sys.argv) > 1:
        console_args.check_requests()
        Downloader().start()

    # GUI version of the app
    else:
        print('GUI version coming soon...')


if __name__ == '__main__':
    run()



import json
from myredditdl.utils import setup_logger
from myredditdl.defaults import Defaults
from pprint import pprint


class Metadata:
    def __init__(self, debug=False):
        self.defaults = Defaults(debug)
        self.json_file = self.defaults.metadata_file
        self.log = setup_logger(__name__, debug)
        self._metadata_map = {}

    def add_to_map(self, filename: str, metadata: dict) -> None:
        ''' Adds a dictionary entry {filename: {metadata}} to the
            metadata map.

        @params:
            - filename: the filename of the entry
            - metadata: reddit post metadata associated to the given filename
        '''
        self._metadata_map[filename] = metadata

    def get_map(self) -> dict:
        ''' Returns the metadata map composed of one or more
            {filename: reddit metadata}

        @return: dictionary of the following format
                 {filename: {metadata}, filename: {metadata}, ...}

        Note: filenames are the keys of the map
        '''
        return self._metadata_map

    def show_metadata(self, filename: str):
        ''' Prints to the console the Reddit post metadata associated
            with the given @filename if it is found.
        '''
        try:
            with open(self.json_file, 'r', encoding='utf-8') as f:
                data = json.load(f)
                if filename in data.keys():
                    print('\n')
                    pprint(data[filename])
                else:
                    self.log.info(f'Metadata for `{filename}` not found')
        except IOError:
            self.log.info('Database not found. Must download content first')

    def show_link(self, filename: str):
        ''' Prints to the console the Reddit post URL link of the given
            media filename.
        '''
        try:
            with open(self.json_file, 'r', encoding='utf-8') as f:
                data = json.load(f)
                if filename in data.keys():
                    print(f"\nLink: {data[filename]['Link']}")
                else:
                    self.log.info(f'Metadata for `{filename}` not found')
        except IOError:
            self.log.info('Database not found. Must download content first')



import time
import praw
import myredditdl.console_args as console_args
import myredditdl.utils as utils
from myredditdl.config_handler import ConfigHandler


class RedditClient:
    def __init__(self) -> None:
        self.client_time = time.time()
        self.arg_dict = console_args.get_console_args()
        self.logger = utils.setup_logger(__name__, self.arg_dict['debug'])
        self.conf = ConfigHandler()
        self.user_instance = None

    @property
    def section_name(self):
        ''' Returns the current active Reddit client username in uppercase'''
        return self.conf.get_client_active_section()

    @property
    def config(self):
        return self.conf.get_config()

    @property
    def max_depth(self):
        return self.arg_dict['max_depth']

    @property
    def client_upvotes(self) -> 'User upvoted posts':
        ''' Yields a ListingGenerator of the user upvoted posts if the
            user asked for the saved files with the (-U --upvote) flag.
            Otherwise, return None
        '''
        if self.user_instance is None:
            self.logger.warning(
                'No valid Reddit client found. Please use `myredditdl --add-client`')
            exit(1)
        return self.user_instance.upvoted(
            limit=self.arg_dict['max_depth']) if self.arg_dict['upvote'] else None

    @property
    def client_saves(self) -> 'User saved posts':
        ''' Yields a ListingGenerator of the user saved posts if the
            user asked for the saved files with the (-S --saved) flag.
            Otherwise, return None
        '''
        if self.user_instance is None:
            self.logger.warning(
                'No valid Reddit client found. Please use `myredditdl --add-client`')
            exit(1)
        return self.user_instance.saved(
            limit=self.arg_dict['max_depth']) if self.arg_dict['saved'] else None

    def build_reddit_instance(self) -> bool:
        self.logger.info('Building reddit instance...')

        if len(self.section_name) == 0:
            self.logger.warning(
                'No valid clients were found.'
                'Add new reddit clients with the --add-client flag')
            return False

        try:
            self.user_instance = praw.Reddit(
                user_agent='myredditdl',
                client_id=self.config[self.section_name]['client_id'],
                client_secret=self.config[self.section_name]['client_secret'],
                username=self.config[self.section_name]['username'],
                password=self.config[self.section_name]['password']).user.me()

        except Exception:
            self.logger.error('Reddit instance build: Failed!')
            return False

        self.logger.debug(
            "Client Build: %s seconds" %
            (time.time() - self.client_time))
        return True

    @classmethod
    def validate_instance(cls, instance: dict) -> bool:
        try:
            instance = praw.Reddit(
                user_agent='myredditdl',
                client_id=instance.get('client_id'),
                client_secret=instance.get('client_secret'),
                username=instance.get('username'),
                password=instance.get('password')).user.me()
            return True

        except Exception:
            print('INFO: instance validator: Failed!')

        return False


if __name__ == '__main__':
    utils.print_warning(utils.DONT_RUN_THIS_FILE)



import configparser
import errno
import os
import sys
from pathlib import Path
import myredditdl.utils as utils


class ConfigHandler:
    def __init__(self) -> None:
        self.log = utils.setup_logger(__name__)
        self.config = configparser.RawConfigParser()
        self.config.read(self.get_config_path())

    def __getitem__(self):
        return self.conf

    def __repr__(self):
        return 'Config(user=%r, prefix=%r, path=%r)' % (
            self.get_client_username(),
            self.get_prefix(),
            self.get_media_path())

    def __str__(self) -> str:
        return self.fmt.format(
            'Configuration:\n',
            'Prefix',
            self.get_prefix(),
            'Path',
            self.get_media_path(),
            'Active Client',
            self.get_client_active_section(),
            'Client id',
            self.get_client_id(),
            'Client secret',
            self.get_client_secret(),
            'Username',
            self.get_client_username())

    def __print__(self, request=None) -> None:
        print(self.__str__())

    @property
    def fmt(self):
        return ('{}\n'
                '{:6} = {}\n'
                '{:6} = {}\n\n'
                '{:13} = {}\n'
                '{:13} = {}\n'
                '{:13} = {}\n'
                '{:13} = {}\n')

    def get_config_path(self):
        return utils.CFG_FILENAME

    def get_config(self):
        return self.config

    def get_config_sections(self):
        return self.config.sections()

    def get_available_reddit_clients(self) -> list:
        return [sec for sec in self.get_config_sections()
                if sec not in ('DEFAULTS', 'USERS', 'EMPTY_CLIENT')]

    def set_new_current_user(self, section_name: str) -> bool:
        ''' @section_name is the current reddit client username in Upper case
            which points to the current user section name in the config file.

            Example:
            ['USERS'] -> this are section names
            current_user_section_name = RANDOM_USERNAME

            [RANDOM_USERNAME] -> this are section names
            client_id=
            client_secret=
            etc...

            Now random_username will be the new active reddit client
        '''
        if section_name == self.get_client_active_section():
            self.log.info(
                f'{section_name} is the current active Reddit client')
            return False

        self.config.set('USERS', 'current_user_section_name', section_name)
        self.write_config()
        self.log.info(f'Changing {section_name} to be the Reddit client')
        return True

    def add_client(self, client: dict) -> bool:
        ''' Receives a client in the form of:

            {section: section name,
             id: client_id,
             secret: client_secret,
             username: username,
             password: password,
            }

            and adds the client to the config file.

        @return: True if the client was added; Flase otherwise
        '''
        section_name = client.get('section')
        if not self.config.has_section(section_name):
            self.config.add_section(section_name)
            self.config.set(section_name, 'client_id', client.get('client_id'))
            self.config.set(
                section_name,
                'client_secret',
                client.get('client_secret'))
            self.config.set(section_name, 'username', client.get('username'))
            self.config.set(section_name, 'password', client.get('password'))
            if self.get_client_active_section() == 'EMPTY_CLIENT':
                self.set_new_current_user(section_name)

            self.write_config()
            self.log.info(
                f"{client.get('username')} successfully added as a client")
            return True

        self.log.info('That client already exists.')
        return False

    def write_config(self) -> bool:
        ''' Config file writer function. Only this function should be used
            to write to the file. To avoid having different instances of configuration
            that could lead to an overwritten config file.

        @return: True if the config file was found and written.
        '''
        try:
            with open(self.get_config_path(), 'w') as configfile:
                self.config.write(configfile)
                return True

        except FileNotFoundError:
            self.log.debug('Write config failed!')
            return False

    def get_default_media_path(self) -> str:
        ''' Returns the location of the user $HOME/Pictures/{reddit_username}_reddit/
            media folder.
        '''
        pictures = os.path.expanduser(f'~{os.sep}Pictures{os.sep}')
        return pictures + self.get_client_active_section() + '_reddit' + os.sep

    def get_client_active_section(self) -> str:
        ''' Reddit client section name of the currently active reddit client'''
        return self.config.get('USERS', 'current_user_section_name')

    def get_client_id(self) -> str:
        ''' Client id of the currently activated reddit client'''
        return self.config.get(self.get_client_active_section(), 'client_id')

    def get_client_secret(self) -> str:
        ''' Client secret of the currently activated reddit client'''
        return self.config.get(
            self.get_client_active_section(),
            'client_secret')

    def get_client_username(self) -> str:
        ''' Client username of the currently activated reddit client'''
        return self.config.get(self.get_client_active_section(), 'username')

    def get_prefix(self) -> str:
        ''' Currently set prefix option'''
        return self.config.get('DEFAULTS', 'prefix')

    def get_media_path(self):
        ''' Returns the current set path where media will be downloaded to.'''
        return self.config.get('DEFAULTS', 'path')

    def get_valid_prefix_options(self) -> set:
        ''' Returns a set of valid filenames prefix configurations.'''
        return {'subreddit',
                'username',
                'subreddit_username',
                'username_subreddit'}

    def set_prefix_option(self, prefix: str) -> bool:
        ''' Receives a prefix option in the form of a string:
            Example: subreddit_username
            Example: username

            Note: see @get_valid_prefix_options() for valid prefixes
        '''
        if prefix == self.get_prefix():
            self.log.info(f'{prefix} is already the current prefix option')
            return False

        elif prefix in self.get_valid_prefix_options():
            self.config.set('DEFAULTS', 'prefix', prefix)
            self.write_config()
            self.log.info(f'Prefix changed to: {prefix}')
            return True

        else:
            self.log.error(utils.INVALID_CFG_OPTION_MESSAGE)
            return False

    def set_media_path(self, path: str) -> bool:
        ''' Changes the destination folder for downloaded media
            to @path
        '''
        if path.lower() == 'default':
            path = self.get_default_media_path()

        elif self.is_path_creatable_or_exists(path):
            path = self.sanitize_path(path)
            path = path if path.endswith(os.sep) else path + os.sep
        else:
            self.log.info(
                f"cannot create directory '{path}': No such file or directory")
            return False

        self.config.set('DEFAULTS', 'path', path)
        self.write_config()
        self.log.info(f'media path set to: {path}')
        return True

    def sanitize_path(self, path: str) -> str:
        ''' Cleans up given @path from --path flag
            This function attempts to handle relative paths but definetly needs
            further testing.
        '''
        root = Path.home() if path.startswith(str(Path.home())) else Path.cwd()
        parts = Path(path).parts
        for part in parts:
            if part == '..':
                root = root.parent

        unwanted_chars = ['..', os.sep, str(root.anchor)]
        unwanted_chars.extend(str(root).split(os.sep))
        unwanted_chars = set(unwanted_chars)
        return str(
            root) + ''.join([os.sep + p for p in parts if p not in unwanted_chars])

    def is_path_creatable_or_exists(self, path: str) -> bool:

        try:
            return self.is_path_valid(path) and (
                os.path.exists(path) or self.is_path_creatable(path))

        except OSError:
            return False

    def is_path_creatable(self, path: str) -> bool:
        dirname = os.path.dirname(path) or os.getcwd()
        return os.access(dirname, os.W_OK)

    # Credits to: stackoverflow.com/questions/9532499

    def is_path_valid(self, path: str) -> bool:
        try:
            if not isinstance(path, str) or not path:
                return False

            _, path = os.path.splitdrive(path)

            root_dirname = os.environ.get('HOMEDRIVE', 'C:') \
                if sys.platform == 'win32' else os.path.sep
            assert os.path.isdir(root_dirname)

            root_dirname = root_dirname.rstrip(os.path.sep) + os.path.sep

            for pathname_part in path.split(os.path.sep):
                try:
                    os.lstat(root_dirname + pathname_part)

                except OSError as exc:
                    if hasattr(exc, 'winerror'):
                        if exc.winerror == utils.ERROR_INVALID_NAME:
                            return False
                    elif exc.errno in {errno.ENAMETOOLONG, errno.ERANGE}:
                        return False

        except TypeError as exc:
            return False
        else:
            return True


if __name__ == '__main__':
    # TODO: change this message
    print('dont run this.')



import time
import requests

import myredditdl.console_args as console_args
import myredditdl.utils as utils
from myredditdl.defaults import Defaults
from myredditdl.file_handler import FileHandler
from myredditdl.item import Item
from myredditdl.reddit_client import RedditClient
from myredditdl.metadata_handler import Metadata


class Downloader(RedditClient):
    def __init__(self):
        super().__init__()
        self.args = console_args.get_console_args()
        self.log = utils.setup_logger(__name__, self.args['debug'])
        self.item = None
        self.valid_domains = None
        self.items_downloaded = 0
        self.items_iterated = 0
        self.items_skipped = 0
        self.defaults = Defaults(self.args['debug'])
        self.file_handler = FileHandler()
        self.metadata_handler = Metadata()

    def __str__(self) -> str:
        return (
            '\n---- Results ----\n'
            f'Items Iterated:   {self.items_iterated}\n'
            f'Items Skipped:    {self.items_skipped}\n'
            f'Items Downloaded: {self.items_downloaded}\n'
            f'Total time:       {(time.time() - self.client_time)} seconds\n')

    @property
    def get_args(self):
        ''' Command-line arguments'''
        return self.args

    def get_valid_domains(self) -> set:
        ''' Downloadable domains.
            If the --no-nsfw flag was given the only return SFW domains
            else return a union of both NSFW and SFW domains
        '''
        if self.args['no_nsfw']:
            return self.__sfw_domains()
        return self.__sfw_domains() | self.__nsfw_domains()

    def __sfw_domains(self) -> set:
        ''' Returns a set of SFW domains'''
        return {'v.redd.it',
                'i.redd.it',
                'i.imgur.com',
                'gfycat.com',
                'streamable.com',
                'reddit.com',
                'imgur.com'}

    def __nsfw_domains(self) -> set:
        ''' Returns a set of NSFW domains'''
        return {'redgifs.com', 'erome.com'}

    def download_limit_reached(self) -> bool:
        ''' Checks if the download limit has been reached.
            Download limit it's dictated by the --limit [int] flag.
            If the number of items downloaded reaches the limit set by the user,
            then return True.
            If no --limit flag was given this function always returns False
        '''
        if self.args['limit'] and self.items_downloaded >= self.args['limit']:
            return True
        return False  # Download limit hasn't been reached or no limit was given

    def is_valid_domain(self) -> bool:
        ''' Returns True if the domain of the current item is valid. False otherwise'''
        return self.item.get_domain() in self.valid_domains

    def is_valid_subreddit(self) -> bool:
        ''' Checks whether the current item subreddit is valid or not.
            If the user gave the --sub flag then only the items that belongs
            to the given subreddit(s) will be valid. If no --sub flag
            was given then all the items will have valid subreddits
        '''
        if self.args['sub'] is None:
            return True
        return self.item.get_subreddit().lower() in self.args['sub']

    def can_download(self) -> bool:
        ''' Checks if the item can be downloaded or not
            Invalid Items that can't be downloaded:
                - Empty items
                - Gallery item and --no-gallery flag given
                - Video item and --no-video flag given
                - Image item and --only-video flag given
                - File has already been downloaded before

        @return: True if can download; False otherwise
        '''
        if len(self.item) == 0:  # empty item
            return False
        if len(self.item) > 1 and self.args['no_gallery']:
            return False
        if self.args['only_video'] and not self.item.is_video():
            return False
        if self.args['no_video'] and self.item.is_video():
            return False
        if self.file_handler.file_exists():
            self.log.info('Item exists')
            return False
        return True

    def download_item(self) -> None:
        ''' Downloads the item from using the downloable url of the item'''
        data = self.get_data()
        for i in range(len(self.item)):
            r = requests.get(data[i].get('url'))
            with open(data[i].get('path'), 'wb') as f:
                f.write(r.content)
                self.log.info(
                    f'Item added: {self.file_handler.get_filename(i)}')

    def __check_if_metadata_save(self) -> bool:
        ''' Checks if the user gave the --no-metadata flag and if the
            flag wasn't given then add the item data to the metadata_handler
            data structure.
        '''
        if not self.args['no_metadata']:
            filename = self.file_handler.get_filename(0)
            self.metadata_handler.add_to_map(filename,
                                             self.item.get_metadata())
            return True
        return False

    def get_data(self) -> list:
        ''' Returns a list that contains a dictionary of url and filepath pairs.
            The reason for it being a list is that some reddit post are gallery media
            items (i.e more than one media file per item), hence in the case of a gallery
            item we need to iterate through all the gallery media url/paths.

        @return: List of dictionary pairs of url:filepath
                (can be more than one pair per list)
        '''
        return [{'url': self.item.get_media_url()[i],
                 'path': self.file_handler.absolute_path[i]}
                for i in range(len(self.item))]

    def __iterate_items(self, items: 'Upvoted and/or Saved posts') -> None:
        ''' Iterate through the user reddit upvoted and/or saved @items
            and call for download the @items that meet the criteria for download.

        @param:
            - items: User Reddit Upvoted and/or Saved media posts

        '''
        for post_item in items:
            if self.download_limit_reached():
                break
            self.item = Item(post_item)
            self.items_iterated += 1
            if self.is_valid_domain() and self.is_valid_subreddit():
                self.file_handler.set_current_item(self.item)
                if self.can_download():
                    self.download_item()
                    self.__check_if_metadata_save()
                    self.items_downloaded += 1
            else:
                self.items_skipped += 1

    def start(self) -> None:
        ''' Main downloader driver'''
        if not self.build_reddit_instance():
            return

        self.valid_domains = self.get_valid_domains()

        if self.args['upvote']:
            self.__iterate_items(self.client_upvotes)
        elif self.args['saved']:
            self.__iterate_items(self.client_saves)
        else:
            self.log.error(utils.MISSING_DOWNLOAD_SOURCE)

        if self.args['debug']:
            self.file_handler.debug_clean()

        if not self.args['no_metadata']:
            self.file_handler.write_metadata(self.metadata_handler.get_map())

        self.log.info(self)


if __name__ == '__main__':
    print(utils.DONT_RUN_THIS_FILE)



import myredditdl.utils as utils
from myredditdl.config_handler import ConfigHandler
from myredditdl.reddit_client import RedditClient


class Terminal:
    def __init__(self):
        self.log = utils.setup_logger(__name__)
        self.config = ConfigHandler()

    def display_setup_header(self) -> None:
        instructions = utils.DEVELOPER_APP_INSTRUCTIONS
        header = '-' * 20 + ' SETUP CONFIGURATOR ' + '-' * 20
        print('\n' + header)
        print(
            'For information'
            ' on how to setup your own developer credentials to use myredditdl,\n'
            'Please refer to',
            instructions)

        print(
            '\nInput your reddit developer credentials below. Make sure they are correct.')
        print('-' * len(header))

    def new_client_prompt(self) -> dict:
        ''' Prompts the user for new client information and returns the
            information as a dictionary.
        '''
        client_id = input('1. Client Id: ').strip(' ')
        client_secret = input('2. Client secret: ').strip(' ')
        username = input('3. Username: ').strip(' ')
        password = input('4. Password: ').strip(' ')
        print('\n')

        return {'section': username.upper(),
                'client_id': client_id,
                'client_secret': client_secret,
                'username': username,
                'password': password}

    def client_setup(self, request=None) -> None:
        ''' Sets up new Reddit client if the given credentials are valid
            Reddit instances. If valid, then go ahead an add the client.
            If invalid; exit the program.
        '''
        self.display_setup_header()
        new_client_instance = self.new_client_prompt()

        if RedditClient.validate_instance(new_client_instance):
            self.config.add_client(new_client_instance)
        else:
            self.log.warning('Invalid reddit instance provided')
            exit(0)

    def change_client_prompt(self, options: dict) -> None:
        print('\n Valid Reddit Clients:\n')
        for k, v in options.items():
            print(f'{k}. {v}')

        res = ''
        while res not in options.keys():
            res = input('\nPlease, chose a client to change to: ')

        if res == str(len(options)):
            self.log.info('Thank you for using myredditdl')
            exit(0)
        self.config.set_new_current_user(options.get(res))

    def get_clients_options(self) -> dict:
        clients = self.config.get_available_reddit_clients()
        clients.append('Exit program')
        return {str(index): client for index, client in enumerate(clients, 1)}

    def change_client(self, request=None):
        options = self.get_clients_options()
        self.change_client_prompt(options)

    def change_path(self, request: str):
        self.config.set_media_path(request)

    def change_prefix(self, request: list):
        self.config.set_prefix_option('_'.join(request))



import os
import logging
from pathlib import Path

# TODO: refactor this entire file.


def setup_logger(module: str, debug=False):
    logger = logging.getLogger(module)
    if not logger.handlers:
        logger.setLevel(logging.DEBUG if debug else logging.INFO)

        fh = logging.FileHandler('debug.log', 'a')
        fh.setLevel(logging.DEBUG)

        sh = logging.StreamHandler()
        sh.setLevel(logging.DEBUG if debug else logging.INFO)

        fh_formatter = logging.Formatter(
            '%(levelname)s: %(name)s : %(message)s')
        sh_formatter = logging.Formatter('%(levelname)s: %(message)s')
        fh.setFormatter(fh_formatter)
        sh.setFormatter(sh_formatter)

        logger.addHandler(fh)
        logger.addHandler(sh)
        logger.propagate = False

    if module == 'reddit_client':
        logger.debug('-' * 60)

    return logger


# Project source directory: myredditdl/myredditdl/
SRC_DIR = str(Path(__file__).parent) + os.sep
# Project parent directory: myreddit-dl/
PROJECT_PARENT_DIR = str(Path(__file__).parent.parent) + os.sep

# config file
CFG_FILENAME = SRC_DIR + 'config.ini'
CFG_PREFIX_DEFAULT = 'subreddit'


DEVELOPER_APP_INSTRUCTIONS = ('https://github.com/emanuel2718/myredditdl'
                              '/blob/master/PRE_INSTALL.md')


INVALID_CFG_OPTION_MESSAGE = ('Invalid save option.\n\n'
                              'Valid Options:\n'
                              '1. --config-prefix username\n'
                              '2. --config-prefix subreddit\n'
                              '3. --config-prefix subreddit username\n'
                              '4. --config-prefix username subreddit\n')

ERROR_INVALID_NAME = 123


# Domains and usefuls urls
REDDIT_GALLERY_URL = 'https://www.reddit.com/gallery/'
NSFW_DOMAINS = {'redgifs.com', 'erome.com'}
SFW_DOMAINS = {'v.redd.it', 'i.redd.it', 'i.imgur.com',
               'gfycat.com', 'streamable.com', 'reddit.com', 'imgur.com'}

VIDEO_DOMAINS = {'redgifs.com', 'gfycat.com', 'v.redd.it'}

SUPPORTED_MEDIA_FORMATS = ('.jpg', '.png', '.jpeg', '.gif', '.gifv', '.mp4')
EXTENSION_STRING = '.jpg.png.jpeg.gif.gifv.mp4'

# Usefuls messages
DONT_RUN_THIS_FILE = ('This file is not intended to be run by itself. '
                      'See myredditdl -h for more information')

USER_NOT_FOUND = 'User not found. Possible error in the `config.ini` file.\n'

MISSING_STORING_METHOD = (
    'Required argument missing.\n\n'
    'Please, Specify storing method.\n\n'
    'Options (--by-user or --by-subreddit): \n\n'
    '--by-subreddit  store media with post subreddit '
    'name in front of filename\n'
    '--by-user\tstore media with post author name in front '
    'of filename\n')

MISSING_DOWNLOAD_SOURCE = ('Required argument missing.\n\n'
                           'Please, specify source of media to download\n\n'
                           'Options (-U and/or -S): \n\n'
                           '-U   download upvoted media\n'
                           '-S   download saved media\n')


def get_valid_prefix_options():
    return (
        'subreddit',
        'username',
        'subreddit_username',
        'username_subreddit')


def print_metadata(data: dict) -> None:
    data = data.replace("{", '').replace("}", '').replace("'", '').split(',')
    print('\n\t[METADATA FOUND]\n')
    for i in data:
        print(i.lstrip(' '))
    print('\n')


if __name__ == '__main__':
    print('Dont run this file')



import argparse
import textwrap
import myredditdl.utils as utils
from myredditdl.terminal import Terminal
from myredditdl.config_handler import ConfigHandler
from myredditdl.metadata_handler import Metadata


def __mapped_requests():
    ''' The request flags with the associated function calls as key/value pairs'''
    args = get_console_args()
    terminal = Terminal()
    metadata_handler = Metadata(args['debug'])
    return {'add_client': terminal.client_setup,
            'change_client': terminal.change_client,
            'show_config': ConfigHandler().__print__,
            'path': terminal.change_path,
            'prefix': terminal.change_prefix,
            'get_metadata': metadata_handler.show_metadata,
            'get_link': metadata_handler.show_link}


def check_requests():
    ''' Checks if user requested configuration or metadata requests'''
    args = get_console_args()
    for request, func_call in __mapped_requests().items():
        if args[request]:
            func_call(args[request])
            exit(0)


def get_console_args():
    parser = argparse.ArgumentParser(
        description='Reddit upvoted & saved media downloader',
        usage='myredditdl [-h] [REQUIRED] [OPTIONS]',
        formatter_class=argparse.RawTextHelpFormatter)

    required_group = parser.add_argument_group(
        'Required Arguments')

    config_group = parser.add_argument_group('Configuration')
    metadata_group = parser.add_argument_group('Metadata')

    required_group.add_argument(
        '-U',
        '--upvote',
        action='store_true',
        help="Download upvoted media",
        required=False)

    required_group.add_argument(
        '-S',
        '--saved',
        action='store_true',
        help="Download saved media",
        required=False)

    config_group.add_argument(
        '--add-client',
        action='store_true',
        help=textwrap.dedent('''\
        add new reddit account

        '''),
        required=False)

    config_group.add_argument(
        '--change-client',
        action='store_true',
        help=textwrap.dedent('''\
        change to another valid existing reddit client (account)

        '''),
        required=False)

    config_group.add_argument(
        '--prefix',
        type=str,
        nargs='*',
        default=None,
        help=textwrap.dedent('''\
        set filename prefix (post author username and/or post subreddit name)

        Options:

            --prefix username           ---> username_id.extension
            --prefix username subreddit ---> username_subreddit_id.extension
            --prefix subreddit username ---> subreddit_username_id.exension
            --prefix subreddit          ---> subreddit_id.exension

        Default: subreddit_username ---> subreddit_username_id.extension

        '''),
        metavar='OPT',
        required=False)

    config_group.add_argument(
        '--path',
        type=str,
        default=None,
        help='set the path to the folder were media will be downloaded to',
        metavar='PATH',
        required=False)

    config_group.add_argument(
        '--show-config',
        action='store_true',
        help="print current configuration to the terminal",
        required=False)

    parser.add_argument(
        '-v',
        '--version',
        help='displays the current version of myredditdl',
        action='store_true')

    parser.add_argument(
        '-debug',
        '--debug',
        action='store_true',
        help='debug flag',
        required=False)

    parser.add_argument(
        '--limit',
        type=int,
        help="limit the amount of media to download (default: None)",
        required=None)

    parser.add_argument(
        '--max-depth',
        type=int,
        help="maximum amount of posts to iterate through",
        metavar='DEPTH',
        required=False)

    parser.add_argument(
        '--no-gallery',
        action='store_true',
        help="don't download galleries (posts with more than one image/video)",
        required=False)

    parser.add_argument(
        '--no-video',
        action='store_true',
        help="don't download video files",
        required=False)

    parser.add_argument(
        '--only-video',
        action='store_true',
        help="only download video files",
        required=False)

    parser.add_argument(
        '--no-nsfw',
        action='store_true',
        help="disable NSFW content download",
        required=False)

    parser.add_argument(
        '--sub',
        type=str,
        nargs='*',
        help="only download media that belongs to the given subreddit(s)",
        metavar='SUBREDDIT',
        required=None)

    parser.add_argument(
        '--clean-debug',
        action='store_true',
        help="remove all debug files",
        required=False)

    metadata_group.add_argument(
        '--no-metadata',
        action='store_true',
        help="don't save metadata for the downloaded media",
        required=False)

    metadata_group.add_argument(
        '--get-metadata',
        type=str,
        default=None,
        help="print reddit metadata of given FILE",
        metavar='FILE',
        required=False)

    metadata_group.add_argument(
        '--get-link',
        type=str,
        help="print reddit link of given FILE",
        metavar='FILE',
        required=False)

    metadata_group.add_argument(
        '--get-title',
        type=str,
        help="print post title of given FILE",
        metavar='FILE',
        required=False)

    metadata_group.add_argument(
        '--delete-database',
        action='store_true',
        help="delete the database of the current active reddit client user",
        required=False)

    return vars(parser.parse_args())


if __name__ == '__main__':
    utils.DONT_RUN_THIS_FILE



#!/usr/bin/env python

from setuptools import setup
import myredditdl


# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(name='myredditdl',
      version=myredditdl.__version__,
      description="Reddit upvoted and saved media downloader",
      long_description=long_description,
      long_description_content_type='text/markdown',
      classifiers=[
          "Development Status :: 3 - Alpha",
          "Environment :: Console",
          "Intended Audience :: Developers",
          "Programming Language :: Python :: 3",
          "Programming Language :: Python :: 3.6",
          "Programming Language :: Python :: 3.7",
          "Programming Language :: Python :: 3.8",
          "Programming Language :: Python :: 3.9",
          "Topic :: Documentation",
      ],
      keywords='myredditdl reddit downloader command line',
      author='Emanuel Ramirez Alsina',
      author_email='eramirez2718@gmail.com',
      maintainer='Emanuel Ramirez Alsina',
      maintainer_email='eramirez2718@gmail.com',
      url='https://github.com/emanuel2718/myredditdl',
      python_requires='>=3.6',
      license='MIT',
      packages=['myredditdl'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'requests',
          'praw',
      ],
      entry_points={
          'console_scripts': [
              'myredditdl = myredditdl.myredditdl:run',
          ]
      }
      )



import os
import json
import shutil
import myredditdl.utils as utils
from urllib.parse import urlparse
from myredditdl.defaults import Defaults
from myredditdl.console_args import get_console_args


class FileHandler:
    def __init__(self):
        self.log = utils.setup_logger(__name__, True)
        self.item = None
        self.defaults = Defaults(get_console_args()['debug'])

    @property
    def media_path(self) -> str:
        return self.defaults.media_path

    @property
    def absolute_path(self) -> list:
        return [self.media_path + self._filename(url, str(i))
                for i, url in enumerate(self.item.get_media_url())]

    def _filename(self, url: str, index='') -> str:
        ''' Returns only the filename with extension of the current item.
            The url is needed because we parse the file extension from the url.
            The index is needed in order to differentiate the gallery items.

        @params:
            - url:   media url of the current set reddit item
            - index: number that will be added after the id of the current item

        @return: filename with extension of the current item
        '''
        extension = self.get_extension(url)
        prefix = self.defaults.current_prefix
        index = '' if index == '0' else index
        return self.prefix_map().get(prefix) + self.item.get_id() + index + extension

    def set_current_item(self, item: 'Reddit post item'):
        ''' Helper function to set the current reddit @item'''
        self.item = item

    def file_exists(self):
        if os.path.isfile(self.absolute_path[0]):
            return True
        else:
            self.create_path()
            return False

    def create_path(self):
        if os.path.isdir(self.media_path):
            return
        try:
            os.makedirs(self.media_path)
            self.log.info(f'Path created: {self.media_path}')
        except Exception:
            self.log.bebug(f'invalid path: {self.media_path}')

    def get_extension(self, url: str) -> str:
        ''' Parses and returns the file extension based on the url

        @param:
            - url: media url of the current set item

        @return: file extension of the current set item
        '''
        try:
            parsed = urlparse(url)
            _, ext = os.path.splitext(parsed.path)

        except Exception:
            self.log.error(f'Getting file extension of {url}')

        return ext if not ext.endswith('.gifv') else '.mp4'

    def prefix_map(self) -> dict:
        ''' --prefix flags and filename prefixed as key-value pairs'''
        username = self.item.get_author()
        subreddit = self.item.get_subreddit()
        return {'username': username + '_',
                'subreddit': subreddit + '_',
                'subreddit_username': subreddit + '_' + username + '_',
                'username_subreddit': username + '_' + subreddit + '_',
                }

    def get_filename(self, index: int) -> str:
        _, filename = os.path.split(self.absolute_path[index])
        return filename

    def write_metadata(self, metadata_map: dict) -> bool:
        ''' Dump the @metadata_map data into the metadata file

        @param:
            - metadata_map: dictionary of {filename: metadata, filename: metadata, ...}
        '''
        try:
            with open(self.defaults.metadata_file, 'r', encoding='utf-8') as f:
                data = json.load(f)
        except Exception:
            self.log.debug(
                f'Metadata file created: {self.defaults.metadata_file}')
            if metadata_map:
                data = {
                    f'{list(metadata_map.keys())[0]}': f'{list(metadata_map.values())[0]}'}
            else:
                return
        for filename, metadata in metadata_map.items():
            if filename not in data:
                data[filename] = metadata
                self.log.debug(f'Added Metadata for: {filename}')
            else:
                self.log.debug(f'Metadata for {filename} is already saved.')

        with open(self.defaults.metadata_file, 'w') as f:
            json.dump(data, f, indent=4)

    def debug_clean(self) -> None:
        try:
            shutil.rmtree(self.defaults.debug_media_dir)
            self.log.debug('Debug media folder removed')

        except FileNotFoundError:
            self.log.debug('No debug media folder found')

        try:
            os.remove(self.defaults.metadata_file)
            self.log.debug(
                f'Metadata file removed: {self.defaults.metadata_file}')

        except FileNotFoundError:
            self.log.debug('No debug metadata file found')


if __name__ == '__main__':
    utils.print_warning(utils.DONT_RUN_THIS_FILE)
